import re

from log_constants import LogConstants


class URLBuilder(object):
    def __init__(self, protocol, host, port, regex_list):
        self.__protocol = protocol
        self.__host = host
        self.__port = port
        self.__url_prefix = protocol + '://' + host
        self.__regex_list = regex_list
        self.__fix_host_regex = re.compile('^http://[^/]*')

    def __fix_common_problems(self, request_line):
        if request_line.startswith('http://'):
            return self.__fix_host_regex.sub('', request_line)
        else:
            return request_line

    def build(self, request_data):
        request_line = request_data[LogConstants.REQUESTLINE].split(' ')

        if len(request_line) < 2:
            raise IOError('Ignoring request line %s', request_data[LogConstants.REQUESTLINE])

        # check and fix broken request paths
        request_path = self.__fix_common_problems(request_line[1])

        url = self.__url_prefix + request_path
        for regex_tuple in self.__regex_list:
            url = regex_tuple[0].sub(regex_tuple[1], url)
        return url


import logging
import os

from hurry.filesize import size

from log_constants import LogConstants


class Inspector(object):
    def __init__(self):
        self.__count = 0
        self.__failed = 0
        self.__status_match = 0
        self.__length_match = 0
        self.__size = 0

    def __add__(self, other):
        self.__count += other.__count
        self.__failed += other.__failed
        self.__status_match += other.__status_match
        self.__length_match += other.__length_match
        self.__size += other.__size
        return self

    def __str__(self):
        result = 'Requests:\t\t' + str(self.__count) + os.linesep
        result += os.linesep

        failed_perc = 0
        if self.__count > 0:
            failed_perc = (self.__failed * 100) / self.__count
        result += 'Failed requests:\t' + str(failed_perc) + '% (' + str(self.__failed) + ' from ' + str(
            self.__count) + ')' + os.linesep

        status_perc = 0
        if self.__count > 0:
            status_perc = (self.__status_match * 100) / self.__count
        result += 'Status code matched:\t' + str(status_perc) + '% (' + str(
            self.__status_match) + ' from ' + str(self.__count) + ')' + os.linesep

        result += 'Content transferred:\t' + size(self.__size) + ' (' + str(self.__size) + ' bytes)' + os.linesep
        result += 'Size matched:\t\t' + str(self.__length_match) + ' from ' + str(self.__count)

        return result

    def inspect_fail(self, thread_name, url, reason):
        self.__count += 1
        self.__failed += 1

        logging.error('[%s] Request %s failed with reason %s', thread_name, url, str(reason))

    def inspect_succeed(self, thread_name, url, log_data, response):
        self.__count += 1

        code = str(response.status_code)
        if code == log_data[LogConstants.STATUSCODE]:
            self.__status_match += 1

        content = response.text
        length = len(content)
        self.__size += length
        if str(length) == log_data[LogConstants.BYTES]:
            self.__length_match += 1

        logging.debug('[%s] URL: %s Status: %s Length: %i', thread_name, url, code, length)


from exceptions import IOError
import os
import re
import ConfigParser

from config_constants import ConfigConstants


class ConfigReader(object):
    def __init__(self, config_file):
        self.config = {}
        if config_file is not None:
            if os.path.isfile(config_file):
                self.__read_config_file(config_file)
            else:
                raise IOError('File ' + config_file + ' not found')

    def __read_config_file(self, config_file):
        cfgparser = ConfigParser.SafeConfigParser()
        cfgparser.optionxform = str
        cfgparser.read(config_file)
        self.__copy_value(cfgparser, 'General', 'LogFormat', ConfigConstants.LOGFORMAT, True)
        self.__copy_value(cfgparser, 'General', 'Host', ConfigConstants.HOST)
        self.__read_filter(cfgparser, ConfigConstants.FILTER)
        self.config[ConfigConstants.HEADER] = self.__read_dict(cfgparser, 'Header')
        for section in cfgparser.sections():
            if section.startswith('Transform'):
                self.__copy_regex_tuple(cfgparser, section, 'Search', 'Replace', ConfigConstants.TRANSFORM)

    @staticmethod
    def __read_dict(config, section):
        header_dict = {}
        if config.has_section(section):
            for option in config.options(section):
                header_dict[option] = config.get(section, option)
        return header_dict

    def __copy_regex_tuple(self, config, section, arg1, arg2, target):
        if config.has_section(section):
            regex = re.compile(config.get(section, arg1))
            replace = config.get(section, arg2)
            named_tuple = (regex, replace)
            if target in self.config:
                self.config[target].append(named_tuple)
            else:
                self.config[target] = [named_tuple]

    def __read_filter(self, config, section):
        if config.has_section(section):
            cfg = {}
            for option in config.options(section):
                if option == ConfigConstants.FILTERRULE:
                    value = config.get(section, option).strip().lower()
                    self.config[ConfigConstants.FILTERRULE] = not value == 'disallow'
                else:
                    values = config.get(section, option).split(',')
                    cfg[option] = filter(None, map(str.strip, values))
            self.config[section] = cfg

    def __copy_value(self, config, section, option, target, raw=False):
        if config.has_option(section, option):
            self.config[target] = config.get(section, option, raw)

    def merge_dict(self, args):
        for key in args.keys():
            value = args[key]
            if value is not None:
                self.config[key] = args[key]


from setuptools import setup

import replayer


setup(name='replayer',
      version=replayer.__version__,
      description='Replay easily an Apache access.log',
      url='https://github.com/buechele/replayer',
      author='Andreas Buechele',
      author_email='andreas@buechele.org',
      license='MIT',
      packages=['replayer'],
      scripts=['bin/replayer'],
      install_requires=[
          'apachelog',
          'requests',
          'hurry.filesize',
      ],
      platforms='any',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: MIT License',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Topic :: System :: Systems Administration'
      ],
      zip_safe=False)